c***********************************************************************
c     maximum dimensions
c     
c     MAXINT: maximum number of intervals
c     MAXTERM: maximum number of thermal units plus contracts
c     MAXCOMB: maximum number of fuels
c     MAXNAT: maximum number of hydro commodities (types of inflows)
c     MAXNEB: maximum number of reservoirs
c     MAXNAR: maximum number of hydro arcs (including discharge
c             + spillage + transhipment arcs)
C     MANTEN: for maintenances
C     MAXLOADS: for observations of load in an interval (~= hours/interval)
c***********************************************************************
C      integer*4 MAXINT,MAXTERM,MAXCOMB,MAXNAT,
C     $     MAXNEB,MAXNAR, MANTEN, MAXLOADS
C
C      parameter (MAXINT= 200, MAXTERM= 300)
C      parameter (MAXCOMB= 100, MAXNAT= 10)
C      parameter (MAXNEB= 100, MAXNAR= 300)
C      parameter (MANTEN= 100, MAXLOADS=1500)

      include 'HTC_parameter.h'

c***********************************************************************
c     General data about the model
c
c     nint: number of intervals
c     nhores(nint): duration of each interval (hours)
c     ppunta(nint): maximum load (Mw)
C     monot(): discretization of l.d.c.
c     nforma(nint): type of l.d.c
c     nat: number of hydro commodities (types of inflows)
c     probhd(nat-1): probabilities for each type of inflow
c     phdmax: maximum hydraulic power (Mw)
c     escal_potencia: scaling factor for power
c     escal_hores: scaling factor for time
c     escal_energia: scaling factor for energy
c     ordre_preu: scaling factor for prices
c***********************************************************************
      common /general_data/ nhores,ppunta,nforma,phdmax,probhd,
     $     escal_potencia,escal_hores,escal_energia,ordre_preu,nint,nat,
     >     monot

      real*8 nhores(MAXINT),ppunta(MAXINT),phdmax,probhd(MAXNAT),
     $     escal_potencia,escal_hores,escal_energia,ordre_preu    
      
      integer*4 nint,nat,nforma(MAXINT), monot(MAXINT*NQT)


c***********************************************************************
C     Load information
C
C     At present, there is no items here, though it would be interesting
C     to keep monot, ppunta, and some others (maybe in the future)
c***********************************************************************
c     Thermal system data
c
c     nterm: number of thermal units
c     nomterm(nterm): name of the thermal units
c     idtem(nterm): three character identifier (obsolete, maintained 
c                   for Snopt debugging)
c     pmaxterm(nterm): maximum power for each thermal unit (Mw)
c     pminterm(nterm): minimum power for each thermal unit (Mw)
c     te_limits_comb(nterm): tells if th. unit have fuel limitations
c     pmaxlim(nterm): approx. maximum power in case of fuel limitations
c     panes(nterm): failure probability for each thermal unit ([0,1]) 
c     nartterm(nterm): number of operational fuels of each th. unit
c     artterm(t,f): types of fuel used for each th. unit
c     rendiment_g(t,f): efficiency of thermal unit t with fuel f (in %) 
c     potcomprar(t,f): maximum purchase of fuel f by th. unit t (Mwh) 
c     potguardar(t,f): maximum storage of fuel f by th. unit t (Mwh) 
c
c     es_contracte(nterm): tells if it is a thermal unit or a contract
c     cost_lin(nterm): linear cost for contracts
c     cost_quad(nterm): quadratic cost for contracts
c     ini_contract(nterm): initial interval of contract availability
c     fi_contract(nterm): final interval of contract availability
c     
c     ncomb: number of fuels
c     nomcomb(ncomb): names of the fuels
c     preu(ncomb): price of each fuel (ECU/Mwh)
c     totalcomb(ncomb): maximum availability for each fuel (Mwh)
c     apreciacio(ncomb): price apreciation (1= no apreciation/deprec.)
c     preuext: price of external energy (ECU/Mwh)
c     nthrmarticles: total number of fuels used for all the th. units
c                   (contracts use one fuel)
c     
C     A d d e d   m a i n t e n a n c e s   i n f o   a n d   s o   o n
C     nman: number of maintenances
C     minimcomb(ncomb): minimum availability for each fuel (Mwh)
C     thu_man(nman): name of thermal unit with maintenance
C     pmx_man(nman): maximum power allowed during maintenance
C     from(nman): initial week of maint.
C     until(nman): final week of maint.
c***********************************************************************
      common /thermal_system/ pmaxterm,pminterm,rendiment_g,preu,
     $     preuext,apreciacio,panes,potcomprar,potguardar,
     $     cost_lin,cost_quad,pmaxlim,
     $     nterm,nthrmarticles,nartterm,artterm,totalcomb,ncomb,
     $     ncontr,ini_contract,fi_contract,
     $     es_contracte,te_limits_comb,nomcomb,nomterm,idterm,
     >     nman, minimcomb, thu_man, pmx_man, from, until
 
      real*8 pmaxterm(MAXTERM),pminterm(MAXTERM),
     $     rendiment_g(MAXTERM,MAXCOMB),
     $     preu(MAXCOMB),preuext,apreciacio(MAXCOMB),
     $     totalcomb(MAXCOMB),panes(MAXTERM),
     $     potcomprar(MAXTERM,MAXCOMB),potguardar(MAXTERM,MAXCOMB),
     $     cost_lin(MAXTERM),cost_quad(MAXTERM),pmaxlim(MAXTERM),
     >     minimcomb(MAXCOMB), pmx_man(MANTEN)
      
      integer*4 nterm,ncomb,ncontr,artterm(MAXTERM,MAXCOMB),
     $     nartterm(MAXTERM),nthrmarticles,
     $     ini_contract(MAXTERM),fi_contract(MAXTERM),
     >     nman, from(MANTEN), until(MANTEN)

      character*10 nomcomb(MAXCOMB),nomterm(MAXTERM),thu_man(MANTEN)
      character*3 idterm(MAXTERM)
      logical es_contracte(MAXTERM),te_limits_comb(MAXTERM) 
      

c***********************************************************************
c     Bezier curves information for model generation and optimization
c     
c     tbase(nint): base time (hours) at each interval
c     pmin(nint): minimum power related to base time
c     max_ho(nint): maximum deterministic water at each interval
c     min_gt(nint): thermal generation decrease per interval permitted  
c                   over maximum power generation to avoid problems 
c                   with CPE (Bezier)
c     energia(nint): total energy under the CPE
c     e0a,e0b,e0c (nint): external energy function parameters for 
c                   each interval
c     t0a,t0b,t0c (nint): function parameters of the time related to 
c                   the external energy, for each interval
c
c     The following data are local to the optimization stage. They could
c     be converted to local variables and removed from here.
c     
c     MAXBEZ: number of CPE points that define the parametric 
c             Bezier curve
c     e(MAXBEZ): energy component of the Bezier points 
c     p(MAXBEZ): power component of the Bezier points 
c     ehd(MAXBEZ): derivatives of e() with respect to hd (hydro energy)
c     phd(MAXBEZ): derivatives of p() with respect to hd (hydro energy)
c     epg(MAXBEZ): derivatives of e() with respect to pg (thermal energ)
c     ppg(MAXBEZ): derivatives of p() with respect to pg (thermal energ)
c     ee(MAXBEZ): coefficients of the 3rd degree polynomial that 
c                 represents the energy component of the Bezier curve
c     pp(MAXBEZ): coefficients of the 3rd degree polynomial that 
c                 represents the power component of the Bezier curve
c     dehd(MAXBEZ): derivatives of ee() with respect to hd (hydro energ)
c     dphd(MAXBEZ): derivatives of pp() with respect to hd (hydro energ)
c     depg(MAXBEZ): derivatives of ee() with respect to pg (therm energ)
c     dppg(MAXBEZ): derivatives of pp() with respect to pg (therm energ)
c***********************************************************************
      integer*4 MAXBEZ
      parameter (MAXBEZ= 4)

      common /bezier_curves_gen_opt/ tbase,max_ho,min_gt,pmin,energia,
     $     e0a,e0b,e0c,t0a,t0b,t0c,
     $     e,p,ehd,phd,epg,ppg,ee,pp,dphd,dehd,depg,dppg

      real*8 tbase(MAXINT),max_ho(MAXINT),min_gt(MAXINT),
     $     pmin(MAXINT),energia(MAXINT),
     $     e0a(MAXINT),e0b(MAXINT),e0c(MAXINT),
     $     t0a(MAXINT),t0b(MAXINT),t0c(MAXINT),
     $     e(MAXBEZ),p(MAXBEZ),ehd(MAXBEZ),phd(MAXBEZ),
     $     epg(MAXBEZ),ppg(MAXBEZ),ee(MAXBEZ),pp(MAXBEZ),
     $     dphd(MAXBEZ),dehd(MAXBEZ),dppg(MAXBEZ),depg(MAXBEZ)
      

c***********************************************************************
c     Hydro system data
c     
c     
c     The following data should be read from the DataBase
c
c     (units -of volumnes, etc.--- are that of the original data 
c     read from the data base; after their lecture, they are converted
c     to the value that will be used during the optimization phase)
c     
c     neb: number of reservoirs
c     nar: number of arcs (discharge+spillage+transhipment)
c     ncc: number of discharge arcs
c     nvv: number of spillage arcs
c     ntv: nubmer of transhipment arcs
c     conca(nar,2): hydro network topology (for each arc, origin and
c                   destination reservoirs)
c     capmut(nar,nint): mutual capacities of the arcs in m3/s
c     capemb(neb,nint): reservoirs capacities in Hm3
c     vlx(neb): reservoir capacities again in Hm3 (auxiliary vector)
c     aportacio(nat,neb,nint): reservoir inflows in m3/s
c     nomembas(neb): names of the reservoirs
c     embas(neb): three character identifiers for the reservoirs
c     tiparc(nar): types of the arcs (spillage, discharge or transh.)
c     limarc(nat,nar,nint): arc bounds for comm. and interval in m3/s
c     limemb(nat,neb,nint): reservoir capacity bounds for commodity 
c                           and interval in Hm3/s
c     volini(nat,neb): initial volumes for comm. and reservoir in Hm3 
c     volfi(nat,neb): final volumes for comm. and reservoir in Hm3
c     minima(nar,nint): minimum flows at arcs in m3/s
c     minime(neb,nint): minimum capacities at reservoirs in Hm3/s
c     sbx(nar): what's that???: it is required to generate snq(nar)
c     ctx(neb): what's that???: it is required to generate snq(nar)
c     retard(nint): arcs delays
c     cvb,cvl,cvq,cvc,cvcu (neb): coefficients of the fourth degree 
c                                 polynomial head volume function
c     ehe5,ehe4,ehe3,ehe2,ehe1,ehe0 (ncc): coefficients of the 2/4 degree 
c                                 polynomial arc efficiency function
c     
c     The following data are created by HTCOOR and used during the
c     model generation and optimization stages
c     
c     nax: number of arcs in the multiinterval (replicated) h-network
c     nnx: number of nodes in the multiinterval (replicaated) h-network
c     pvi: penalty parameter for spillages at non-full reservoirs
c     xarxa((nar+neb)*nint-neb,2): topology of the replicated network
c                   (origin and destination reservoir for each arc)
c     tiparx(nar+neb)*nint-neb): arc types of the replicated network
c     mnk(nar*nint+neb*(nit-1)): vector of origin nodes for all the
c                   arcs of the multiinterval hydro network
c     ipl(neb*nint+1): vector of pointers in mnk() for each node, 
c                   to the list of arcs that arrive to this node
c     rin(neb*nat): initial volumes accumulated by commodity, for all
c                   the reservoirs
c     rfl(neb*nat): final volumes accumulated by commodity, for all the
c                   reservoirs
c     can(nar*nint): minimum flows of the discharge and pumping arcs
c                   for all the intervals
c     jcl(nar): vector of codes (1,2,3) of the disch. and pumping arcs
c     jeo(nar): vector of origin reservoirs of the disch. and pumping 
c               arcs
c     jed(nar): vector of destination reservoirs of the disch. and 
c               pumping arcs
c     snq(nar): vector of head differences between discharges
c     jll(nar): vector for identifying pumping arcs between those of
c               type b, s or d
c     ivo(nar*nint),lvo(nar*nint): indices to arcs of initial and final
c               volums of the origin reservoirs for discharge and 
c               pumping arcs in the multiinterval network.
c     ivd(nar*nint),lvd(nar*nint): indices to arcs of initial and final
c               volums of the destination reservoirs for discharge and 
c               pumping arcs in the multiinterval network.
c     jca(nar*nint): vector of numbers of the arcs in the multiinterval
c               network of the arcs b, s or d
c     cxv(nar): vector of maximum capacities of the origin reservoirs 
c               for the spillage arcs
c     iov(nar): vector of origin reservoirs for the spillage arcs
c     ivv(nar*nint): vector of spillage arcs in the multiinteral network
c     mfv(nar*nint): vector of arcs with final volumes of the origin
c               reservoirs of the spillage arcs, in the multiinterval
c               network
c     von(neb*(nint+1)): vector derived from minime() used during
c               the optimization stage
c     elm(neb),eqt(neb),eqd(neb),ecq(neb),ecua(neb): auxiliary vectors
c               for computing derivatives during optimization phase
c               at funcon()
c     jabom(nar): points to information about pumping arcs in var "salto"
c     salto(6*nar): coeff. for evaluating net jump between reservoirs
c                (according to IBD expression)
c***********************************************************************
      common /hydro_system_read_data/ capmut,capemb,vlx,aportacio, 
     $     limarc,limemb,volini,volfin, 
     $     minima,minime,sbx,ctx,retard,
     $     cvb,cvl,cvq,cvc,cvcu,salto,
     $     ehe5,ehe4,ehe3,ehe2,ehe1,ehe0,neb,nar,ncc,nvv,ntv,
     $     conca,jabom,nomembas,embas,tiparc

      real*8 capmut(MAXNAR,MAXINT),capemb(MAXNEB,MAXINT-1),vlx(MAXNEB),
     $     aportacio(MAXNAT,MAXNEB,MAXINT), 
     $     limarc(MAXNAT,MAXNAR,MAXINT),limemb(MAXNAT,MAXNEB,MAXINT), 
     $     volini(MAXNAT,MAXNEB),volfin(MAXNAT,MAXNEB),
     $     minima(MAXNAR,MAXINT),minime(MAXNEB,MAXINT), 
     $     sbx(MAXNAR),ctx(MAXNEB),
     $     cvb(MAXNEB),cvl(MAXNEB),cvq(MAXNEB), 
     $     cvc(MAXNEB),cvcu(MAXNEB),
     $     ehe5(MAXNAR),ehe4(MAXNAR),ehe3(MAXNAR),
     $     ehe2(MAXNAR),ehe1(MAXNAR),ehe0(MAXNAR),salto(6*MAXNAR)

      integer*4 neb, nar, ncc, nvv, ntv, 
     $     conca(MAXNAR,2),retard(MAXNAR),jabom(MAXNAR)

      character*1 tiparc(MAXNAR) 
      character*3 embas(MAXNEB+1)
      character*10 nomembas(MAXNEB)


      common/hydro_sys_generate_and_optimize/ rin,rfl,pvi,can,snq,
     $     cxv,von,elm,eqt,eqd,ecq,ecua,
     $     nax,nnx,xarxa,mnk,ipl,jcl,jeo,jed,jll,ivo,ivd,lvo,
     $     lvd,jca,iov,ivv,mfv,tiparx
      
      real*8 pvi,rin(MAXNEB*MAXNAT),rfl(MAXNEB*MAXNAT),
     $     can(MAXNAR*MAXINT),snq(MAXNAR),cxv(MAXNAR),
     $     von(MAXNEB*(MAXINT+1)),
     $     elm(MAXNEB),eqt(MAXNEB),eqd(MAXNEB),ecq(MAXNEB),ecua(MAXNEB)
      
      integer*4 nax,nnx,xarxa((MAXNAR+MAXNEB)*MAXINT-MAXNEB, 2),
     $     mnk(MAXNAR*MAXINT+MAXNEB*(MAXINT-1)),ipl(MAXNEB*MAXINT+1),
     $     jcl(MAXNAR),jeo(MAXNAR),jed(MAXNAR),jll(MAXNAR),
     $     ivo(MAXNAR*MAXINT),ivd(MAXNAR*MAXINT),
     $     lvo(MAXNAR*MAXINT),lvd(MAXNAR*MAXINT),
     $     jca(MAXNAR*MAXINT),iov(MAXNAR),ivv(MAXNAR*MAXINT),
     $     mfv(MAXNAR*MAXINT)

      character*1 tiparx((MAXNAR+MAXNEB)*MAXINT-MAXNEB)


c***********************************************************************
c     Variables to store optimization model in Snopt format
c
c     n: number of variables
c     m: number of constraints
c     nb: number of variables+slacks (nb=n+m)
c     ne: number of nonzero entries in Jacobian (including linear 
c         and nonlinear constraints)
c     nncon: number of nonlinear constraints
c     nnobj: number of nonlinear objective function variables
c     nnjac: number of nonlinear jacobian variables
c***********************************************************************
c      common /snopt_vars/ n,m,nb,ne,nncon,nnobj,nnjac

c      integer*4 n,m,nb,ne,nncon,nnobj,nnjac


c***********************************************************************
c     Variables to store data (indices, some values) required by
c     the funobj and funcon routines
c     
c     sumpmaxterm(nint): sum of the maximum power for all the 
c                      thermal units
c     tipuscomb(nterm*ncomb): codes for types of fuels used by
c                      each thermal unit
c     c(nint): indices to variables of purchase of fuel at each interval
c     r(neterm*ncomb): indices to residual fuel at last interval
c                      for each thermal unit and fuel used
c     hn(nint): indices to variables of hydraulic generation HDki
c     hl(nint): indices to variables of inserted hidraulic generation
c               HIit
c     pg(nint): indices to variables PGji of power generated per
c               thermal unit and interval
c     t(nint): indices to variables Gji of generation per
c               thermal unit and interval
c     iphd(nint): indices to variables PHDi of power generated by
c               stochastic water
c     elemblochd: see routine crea_indexs_hd for an explanation
c     p_ijachd4, p_ijachd8: pointers to ijachd() array (for
c                           addresses of 4 and 8 bytes).
c     p_ijacterm4, p_ijacterm8: pointers to ijacterm() array (for
c                               addresses of 4 and 8 bytes).
c     4_bytes_adresses: boolean to decide whether 4 or 8 bytes 
c                       addresses are being used.
c     maxjac: stores an overestimation of the number of elements in
c             ijachd() and ijacterm()
c     inixpgt: index to beginning of PGtttiii variables in solution
c              vector x(). Used to reoptimize adjusting lower bounds
c              for variables PGtttiii.
***********************************************************************
      common /funobj_funcon_data/ sumpmaxterm,c,r,tipuscomb,hn,hl,pg,t,
     $     p_ijachd4,p_ijacterm4,p_ijachd8,p_ijacterm8,
     $     iphd,elemblochd,maxjac,inixpgt, four_bytes_addresses
      
      real*8 sumpmaxterm(MAXINT)
      integer*4 tipuscomb(MAXTERM*MAXCOMB),c(MAXINT),r(MAXINT),
     $     hn(MAXINT),hl(MAXINT),pg(MAXINT),t(MAXINT),iphd(MAXINT),
     $     elemblochd,maxjac,inixpgt
      integer*4 p_ijachd4,p_ijacterm4
      integer*8 p_ijachd8,p_ijacterm8
      logical four_bytes_addresses

c***********************************************************************
c     Temporal data: these variables will likely be removed in the
c     future when reading from the DataWarehouse.
c
c     ffor: name of the l.d.c forms file
c***********************************************************************

      common /tmp_vars/ ffor

      character*256 ffor
