/****************************************************************************

   SLOEGAT-UPC-SOR

   This is main program for HTCOOR.
   It call the model generation and optimization modules.
   It dynamically allocates some space for the optimization stage.

   The name of the madel is given by the command line, e.g.,
   "$ htcoor MODEL"
   
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "macros.h"
#include "structs.h"

/***** prototypes *****/
void tractar_errors(int);
/**********************/

main(int argc, char *argv[])
{
  int 	  nmodel, nunit, nint, nat, ncomb, nterm, ncontr, neb, ncc, nvv, ntv, nar, iexit;
  int     m,n,nb,nncon,nnobj,nnjac,ne,ne_njac;
  int 	 *ha, *ka, *hs, inform, *iws, lencws, leniws, lenrws, mincws, miniws, minrws;
  double *a, *bl, *bu, *xn, *pi, *rc, obj, *rws, pct_cws, pct_iws, pct_rws; 
  char   *cws, *names;
  int    *inijachd, *ijachd, *ijacterm; 
  Fecha   init_date, *inf_dtes;
  
  if (argc != 2) {
    fprintf(stderr,"Usage: htcoor model\n");
    exit(-1);
  }
  
  iexit= 0;

  /**** STEP 0: Create .hydro file (old .dat file). This is a temporary
	file, besides of the .trm file and the .cmb file. In order to
	proceed, it needs some variables that, at present, will be
	obtained in some "magic" way (but in the future could be easily
	read from the BD
  ****/

   do_magic(&neb, &nar, &nat, &init_date, &inf_dtes);

   iexit=deal_with_basin(argv[argc-1], nat, neb, nar);
   if (iexit != 0) tractar_errors(iexit);

  /************************************************************/
  /* 1st: read general data, thermal system, and hydro system */
  /************************************************************/
  

  /**** Reading the model dimensions. Currently this routine can be
    avoided, since when reading the thermal and hydro systems this 
    dimensions are readed again. However this routine is useful if
    dynamic allocation of vectors is done in the future ****/
  
  nmodel= strlen(argv[argc-1]);
  nunit= 9;
  get_dimensions_(argv[argc-1],&nmodel, &nunit, &nint, &nat, &ncomb, &nterm, 
		  &ncontr, &neb, &ncc, &nvv, &ntv, &nar, &iexit);
  if (iexit != 0) tractar_errors(iexit);
  
  /**** Loading general data and the thermal system of the model 
    in fortran "commons" blocks ****/
  
  get_gen_and_thermal_data_(argv[argc-1],&nmodel,&nunit,&iexit); 
  if (iexit != 0) tractar_errors(iexit);
  
  /**** Loading the hydro system in fortran "commons" blocks ****/
  get_hydro_system_(argv[argc-1],&nmodel,&nunit,&iexit,&init_date,inf_dtes); 
  if (iexit != 0) tractar_errors(iexit);
  

  /************************************************************/
  /*                  2nd: manipulate data                    */
  /************************************************************/  
  
  /**** Manipulate general data and thermal system of the model, and 
    leaves them in their final form. The new values will be used by the 
    optimizer****/
  
  deal_with_gen_and_thermal_data_(&iexit);
  if (iexit != 0) tractar_errors(iexit);
  
  /**** Manipulate hydro system of the model, obtaining its
    final form. The new values will be used by the optimizer****/
  
  deal_with_hydro_system_(&iexit);
  if (iexit != 0) tractar_errors(iexit);
  
  release_bckp();	/* This releases the space occupied by some arrays */

  /************************************************************/
  /*             3rd: generate Snopt model                    */
  /************************************************************/  

  /**** Generate dimensions of the optimization problem. Dimensions
   are stored in common blocks ****/
  generate_problem_dimensions_(&m,&n,&nb,&nncon,&nnobj,&nnjac,&ne,&iexit);
  if (iexit != 0) tractar_errors(iexit);
/*  printf("m=%d\t n=%d\t nb=%d\t nncon=%d\nnobj=%d\t nnjac=%d\t ne=%d\n",
      m,n,nb,nncon,nnobj,nnjac,ne); */

  /**** allocate some vectors required by Snopt ****/
  TRY_ALLOC(a,ne,double);
  TRY_ALLOC(ha,ne,int);
  TRY_ALLOC(ka,n+1,int);
  TRY_ALLOC(bl,nb,double);
  TRY_ALLOC(bu,nb,double);
  TRY_ALLOC(hs,nb,int);
  TRY_ALLOC(xn,nb,double);
  TRY_ALLOC(pi,m,double);
  TRY_ALLOC(rc,nb,double);
  TRY_ALLOC(names,8*nb,char);
  if (m*n<1e+3) { 
    pct_cws= 0.5;
    pct_iws= 0.5;
    pct_rws= 0.5;
  }
  else if (m*n<1e+5) {
    pct_cws= 0.25;
    pct_iws= 0.25;
    pct_rws= 0.25;    
  }
  else if (m*n<1e+7) {
    pct_cws= 0.05;
    pct_iws= 0.05;
    pct_rws= 0.05;    
  }
  else {
    pct_cws= 0.02;
    pct_iws= 0.02;
    pct_rws= 0.02;    
  }
  lencws= (int)((double)(m*n)*pct_cws/8.0);
  leniws= (int)((double)(m*n)*pct_iws);
  lenrws= (int)((double)(m*n)*pct_rws);
  TRY_ALLOC(cws,8*lencws,char);
  TRY_ALLOC(iws,leniws,int);
  TRY_ALLOC(rws,lenrws,double);
  
  /**** fill vectors with model information. required by Snopt ****/
  generate_snopt_model_(&m,&n,&nb,&nncon,&nnobj,&nnjac,&ne,&ne_njac,a,ha,
			ka,bl,bu,hs,pi,xn,names,&iexit);
  if (iexit != 0) tractar_errors(iexit);
  /*  check_mps_(&m,&n,&nb,&ne,a,ha,ka,names,bl,bu); */
  /* this is for writing constraints matrix structure in matlab sparse matrix 
     format */
  /*  {
    int nu=21;
    write_matlab_sparse_matrix_(&m,&n,&ne,ha,ka,&nu);
    exit(0);
  }
  */

  /**** create indices to jacobian for hydro and thermal parts ****/
  TRY_ALLOC(inijachd,ne_njac,int);
  TRY_ALLOC(ijacterm,ne_njac,int);
  TRY_ALLOC(ijachd,ne_njac,int);
  crea_indexs_hd_(inijachd,ijachd,&ne_njac,&iexit);
  if (iexit != 0) tractar_errors(iexit);  
  crea_indexs_term_(ijacterm,&ne_njac,&iexit);
  if (iexit != 0) tractar_errors(iexit);
  

  /************************************************************/
  /*             4th: optimize model with Snopt               */
  /************************************************************/  
  
  /**** perform optimization, dealing with problems due to insufficient
    workspace in the ws vectors ****/
  pct_cws= 0.0;
  pct_iws= 0.0;
  pct_rws= 0.0;
  for(inform=20 ; (inform==42)||(inform==43)||(inform==44)||(inform==20) ; ) {
    optimize_problem_(&m,&n,&nb,&nncon,&nnobj,&nnjac,&ne,a,ha,ka,
		      bl,bu,hs,pi,xn,rc,names,&obj,&mincws,&miniws,&minrws,
		      cws,iws,rws,&lencws,&leniws,&lenrws,&inform,&iexit);
    
    printf("Snopt return code= %d\n",inform);
 
    if (inform == 42) {
      pct_cws += 0.10;
      lencws= (int)((double)(m*n)*pct_cws/8.0)+mincws;
      TRY_REALLOC(cws,8*lencws,char);
    }
    if (inform == 43) {
      pct_iws += 0.10;
      leniws= (int)((double)(m*n)*pct_iws)+miniws;
      TRY_REALLOC(iws,leniws,int);
    }
    if (inform == 44) {
      pct_rws += 0.10;
      lenrws= (int)((double)(m*n)*pct_rws)+minrws;
      TRY_REALLOC(rws,lenrws,double);
    }
    if (inform == 20) {
      pct_iws += 0.15;
      pct_rws += 0.15;
      leniws= (int)((double)(m*n)*pct_iws)+miniws;
      lenrws= (int)((double)(m*n)*pct_rws)+minrws;
      TRY_REALLOC(iws,leniws,int);
      TRY_REALLOC(rws,lenrws,double);
    }
  }

  /************************************************************/
  /*     5th: write summary information of the solution       */
  /************************************************************/
  write_info_optim_(&n,&m,&nncon,&ne,xn,&obj); 
  
}

void tractar_errors(int iexit)
{
  
  switch (iexit) {
  case -1:
    fprintf(stderr,"Error: dimensions can not be obtained. ");
    fprintf(stderr, "Parameters data file is missing.\n");
    break;
  case -2:
    fprintf(stderr,"Error: dimensions can not be obtained.");
    fprintf(stderr, "Fuels data file is missing.\n");
    break;
  case -3:
    fprintf(stderr,"Error: dimensions can not be obtained.");
    fprintf(stderr, "Thermal units data is missing.\n");
    break;
  case -4:
    fprintf(stderr,"Error: dimensions can not be obtained. ");
    fprintf(stderr, "Hydro model data is missing.\n");
    break;
  case -5:
    fprintf(stderr,"Error: too many intervals. Increase MAXINT.\n");
    break;
  case -6:
    fprintf(stderr,"Error: too many hydro commodities. Increase MAXNAT.\n");
    break;
  case -7:
    fprintf(stderr,"Error: too many fuels. Increase MAXCOMB.\n");
    break;
  case -8:
    fprintf(stderr,"Error: too many thermal units. Increase MAXTERM.\n");
    break;
  case -9:
    fprintf(stderr,"Error: a thermal unit uses more than available fuels.\n");
    break;
  case -10:
    fprintf(stderr,"Error: a thermal unit uses an unknown fuel.\n");
    break;
  case -11:
    fprintf(stderr,"Error: in initial of final interval for some contract.\n");
    break;
  case -12:
    fprintf(stderr,"Error: an inflows data file could not be opened for some reservoir.\n");
    break;
  case -13:
    fprintf(stderr,"Error: a commodity volumes data file could not be opened for some reservoir.\n");
    break;
  case -14:
    fprintf(stderr,"Error: file with coefficients of the fourth degree polynomial arc efficiency functions could not be opened.\n");
    break;
  case -15:
    fprintf(stderr,"Error: dimension of vector of pointers to hydro part of jacobian must be increased.\n");
    break;
  case -16:
    fprintf(stderr,"Error: dimension of vector of pointers to thermal part of jacobian must be increased.\n");
    break;
  case -17:
    fprintf(stderr,"Error: Load file is missing.\n");
    break;
  case -18:
    fprintf(stderr,"Error: Not enough hours in load file.\n");
    break;
  case -19:
    fprintf(stderr,"Error: Hydro model data is missing.\n");
    break;
  case -20:
    fprintf(stderr,"Error: Too many reservoirs [get_cdf_inflows]\n");
    break;
  case -21:
    fprintf(stderr,"Error: Not enough space reading inflows.\n");
    break;
  case -22:
    fprintf(stderr,"Error: Can't save time division of horizon.\n");
    break;
  case -23:
    fprintf(stderr,"Error: Can't read time division of horizon.\n");
    break;
  case -24:
    fprintf(stderr,"Error: Dates don't match with length duration in file.\n");
    break;
  case -25:
    fprintf(stderr,"Error: Can't read description of basin.\n");
    break;
  case -26:
    fprintf(stderr,"Error: Can't save file with hydro model data.\n");
    break;
  case -27:
    fprintf(stderr,"Error: Not enough space [getLDCshapes].\n");
    break;
  case -28:
    fprintf(stderr,"Error: Thermal park description file not found [get_thermal_park]\n");
    break;
  case -29:
    fprintf(stderr,"Error: Unknown reservoir identifier\n");
    break;
/***			PATRON: copy & paste these 3 lines
  case -:
    fprintf(stderr,"Error: \n");
    break;
***/

  case -100:
    fprintf(stderr,"Error: not enough memory\n");
  }
  exit(-1);
}

					/* interface with fortran */
void tractar_errors_(int iexit)		/* parameter must be used as */
{					/* "%val(iexit)" */
	tractar_errors(iexit);
}

/*..........................ANTICYCLING...............................*/
#define TRUE 1
#define QUEUE 25
#define MAXCICLO 500
#define EQUAL(x,y) (y!=(double)0.0? (x-y)/y<(double)1.e-14 && (y-x)/y<(double)1.e-14 : (x!=(double)0.0? (x-y)/x<(double)1.e-14 && (y-x)/x<(double)1.e-14 : TRUE))

typedef struct {
	int	cont[QUEUE];
	double	fobj[QUEUE];
	int	b, e;
} Cola;

Cola	Q;

Init_cola()
{
	int	i;

	for (i=0; i<QUEUE; i++)
		Q.cont[i] = 0;
	Q.b = -1;
	Q.e = 0;
}

int Entry_cola(x)
double *x;
{
	int	i, j;

	for (i=0; i<Q.e; i++)
		if (EQUAL(Q.fobj[i], *x)) {
			Q.cont[i]++;
			if (Q.cont[i]>MAXCICLO) {
			   return 0;
			}
			else return 1;
		}
	/* x not found: */
	if (Q.e < QUEUE) {
		Q.fobj[Q.e] = *x;
		Q.cont[Q.e] = 1;
		Q.e++;
	} else {
		Q.b = (Q.b+1)%QUEUE;
		Q.fobj[Q.b] = *x;
		Q.cont[Q.b] = 1;
	}
	return 1;
}

init_cola_() {
	Init_cola();
}

int entry_cola_(x)
double *x;
{
	return Entry_cola(x);
}
